/**
 * FakerWB makes it simple to create any amount of fake data within the Editor.
 * You will be able to generate information like names, addresses, phone numbers,
 * internet related data, companies, images, lorem ipsum, dates, random numbers,
 * financial data, etc.
 *
 * FakerWB is build using faker.js - http://github.com/marak/faker.js/
 *
 * @category  WeBuilder Plugin
 * @package   FakerWB
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2017 Peter Klein
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.0
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     60   function ShowHelp(Sender)
 *    147   function HideCaret(Sender)
 *    158   function InsertMaker()
 *    264   function CheckBox1OnClick(Sender)
 *    277   function CheckBox2OnClick(Sender)
 *    296   function OnSelectItem(Sender)
 *    331   function RunFakerDirect()
 *    343   function FakerDirect(Sender)
 *    357   function OnKeyPress(Sender, key)
 *    373   function OnKeyPress2(Sender, key)
 *    389   function OnWebkitCreated(Sender, Browser)
 *    401   function HideWebkitWindow(Sender)
 *    417   function OnWebkitLoadEnd(Sender, Browser, Frame, Status, Res)
 *    428   function CreateWebkit(Callback)
 *    460   function OnWebkitData(channel, data)
 *    491   function OnWebkitConsoleMessage(Sender, Browser, message, source, line, Res)
 *    500   function RunFaker()
 *    516   function Faker(Sender)
 *    527   function OnExit()
 *    541   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 20
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

var win = null;
var web = null;
var PSForm, CheckBox1, CheckBox2, ListBox1;
var KeysListBox1;
var CallbackAfterCreate;
var fakeMarker;

/**
 * Show Help file.
 *
 * @param Tobject  Sender
 *
 * @return void
 */
function ShowHelp(Sender) {

    PSForm = new TForm(WeBuilder);

    var pfact = Round(PSForm.PixelsPerInch / 120);
	var mleft = 12 * pfact;
	var mtop = 8 *pfact;

    PSForm.ClientWidth = 900 * pfact;
    PSForm.ClientHeight = 600 * pfact;
    PSForm.Position = poScreenCenter;
	PSForm.FormStyle = fsStayOnTop;
    PSForm.Color = clBtnFace; // clMenu;
	PSForm.PixelsPerInch = 120;
    PSForm.Caption = "FakerWB Help";

	// PageControl object
	var pctObj = new TPageControl(WeBuilder);
	pctObj.Parent = PSForm;
	pctObj.Align = alClient;
    pctObj.Font.Color = clWindowText;
    pctObj.Font.Height = -16;
	pctObj.Font.Name = "Tahoma";
	pctObj.AlignWithMargins = true;
	pctObj.Cursor = crHandPoint;

    // 1st TabSheet
	var tabSheet1 = new TTabSheet(WeBuilder);
	tabSheet1.Parent = pctObj;
	tabSheet1.PageControl = pctObj;
	tabSheet1.Caption = "Plugin Usage";

	// 1st TPanel object
 	var pnlWrapper1 = new TPanel(WeBuilder);
	pnlWrapper1.Parent = tabSheet1;
	pnlWrapper1.BorderStyle = bsNone;
	pnlWrapper1.Align = alClient;

	// 1st RichText object
 	var rteObj1 = new TRichEdit(WeBuilder);
	rteObj1.Parent = pnlWrapper1;
	rteObj1.Color = clWhite;
	rteObj1.BorderStyle = bsNone;
	rteObj1.Align = alClient;
	rteObj1.ScrollBars = ssVertical;
	rteObj1.Lines.LoadFromFile(Script.Path + "/Manual.rtf");
	rteObj1.ReadOnly = true;
	rteObj1.Cursor = crArrow;
	rteObj1.OnEnter = "HideCaret";

    // 2nd TabSheet
	var tabSheet2 = new TTabSheet(WeBuilder);
	tabSheet2.Parent = pctObj;
	tabSheet2.PageControl = pctObj;
	tabSheet2.Caption = "Marker Syntax";

	// 2nd TPanel object
 	var pnlWrapper2 = new TPanel(WeBuilder);
	pnlWrapper2.Parent = tabSheet2;
	pnlWrapper2.BorderStyle = bsNone;
	pnlWrapper2.Align = alClient;

	// 2nd RichText object
 	var rteObj2 = new TRichEdit(WeBuilder);
	rteObj2.Parent = pnlWrapper2;
	rteObj2.Color = clWhite;
	rteObj2.BorderStyle = bsNone;
	rteObj2.Align = alClient;
	rteObj2.ScrollBars = ssVertical;
	rteObj2.Lines.LoadFromFile(Script.Path + "/Marker Syntax.rtf");
	rteObj2.ReadOnly = true;
	rteObj2.Cursor = crArrow;
	rteObj2.OnEnter = "HideCaret";

   	PSForm.Showmodal;

	delete PSForm;
}

/**
 * OnEnter callback for use in TRichEdit or TEdit.
 * Hide caret in TRichEdit or TEdit when in ReadOnly Mode
 *
 * @param  object   Sender
 *
 * @return void
 */
function HideCaret(Sender) {
	if (Sender.ReadOnly == true) {
		PSForm.ActiveControl  = Sender.Parent;
	}
}

/**
 * Create modal window for selcting and inserting Faker Mustache marker
 *
 * @return void
 */
function InsertMaker() {

    PSForm = new TForm(WeBuilder);

    var pfact = Round(PSForm.PixelsPerInch / 120);
	var mleft = 12 * pfact;
	var mtop = 8 *pfact;

    PSForm.ClientWidth = 600 * pfact;
    PSForm.ClientHeight = (16 * 10 + 32) * pfact;
    PSForm.Position = poScreenCenter;
    PSForm.BorderIcons = biSystemMenu; //remove maximize & minimize buttons
	PSForm.FormStyle = fsStayOnTop;
    PSForm.Color = clBtnFace;
    PSForm.Font.Color = clWindowText;
    PSForm.Font.Height = -16;
	PSForm.Font.Name = "Tahoma";
	PSForm.PixelsPerInch = 120;
    PSForm.Caption = "Insert Fake data";
    //form.OnResize = "OnResize";

	CheckBox1 = new TCheckBox(WeBuilder);
	CheckBox1.Parent = PSForm;
	CheckBox1.Font.Height = -13;
	CheckBox1.Caption = "Expanded mode";
    CheckBox1.ShowHint = True;
    CheckBox1.Hint = "Expand faker marker immediately";
	CheckBox1.SetBounds(mleft, mtop, 117 * pfact, 17 * pfact);
    CheckBox1.Checked = Script.ReadSetting("Expanded mode", "0");
	CheckBox1.OnClick = "CheckBox1OnClick";
	CheckBox1.OnKeyPress = "OnKeyPress2";

	CheckBox2 = new TCheckBox(WeBuilder);
	CheckBox2.Parent = PSForm;
	CheckBox2.Font.Height = -13;
	CheckBox2.Caption = "Short format";
    CheckBox2.ShowHint = True;
    CheckBox2.Hint = "Use short format without parameters";
	CheckBox2.SetBounds(CheckBox1.Width + mleft * 3 * pfact, mtop, 97 * pfact, 17 * pfact);
    CheckBox2.Checked = Script.ReadSetting("Short format", "0");
	CheckBox2.OnClick = "CheckBox2OnClick";
	CheckBox2.OnKeyPress = "OnKeyPress2";

	if (CheckBox1.Checked) {
		CheckBox2.Checked = true;
	}
	if (CheckBox2.Checked) {
		CheckBox1.Enabled = true;
	}
	else {
		CheckBox1.Enabled = false;
	}

	ListBox1 = new TListBox(WeBuilder);
	ListBox1.Parent = PSForm;
	ListBox1.SetBounds(0, 32 * pfact, PSForm.ClientWidth, PSForm.ClientHeight - 32 * pfact);
	ListBox1.Anchors = akLeft + akTop + akRight + akBottom;
	ListBox1.ItemHeight = 16;
	ListBox1.OnDblClick = "OnSelectItem";
	ListBox1.OnKeyPress = "OnKeyPress";

	// Set ListBox1 as active element when form is displayed
	PSForm.ActiveControl = ListBox1;

	var ini = new TIniFile(Script.Path + "keywords.ini");

	// Shadow table for the keys, since TListBox can only show labels
	KeysListBox1 = new TStringList;

	var SL = new TStringList;

	// Read the section headers (faker namespaces) from ini file
	ini.ReadSections(SL);
	for (var i=0;i<SL.Count;i++) {
		var SS = new TStringList;
		// Read the vars in each section
		ini.ReadSection(SL[i], SS);
		for (var j=0;j<SS.Count;j++) {
			var key = SL[i] + "." + SS[j];
			var val = ini.ReadString(SL[i], SS[j], "");
			if (val != "") {
				val = " - " + val;
			}
			ListBox1.Items.Add(key + val);
			KeysListBox1.Add(key);
		}
		delete SS;
	}
	ListBox1.ItemIndex = 0

	delete SL;
	delete ini;

   	PSForm.Showmodal;

	delete KeysListBox1;
    delete PSForm;
}

/**
 * Callback handler for clicks on Checkbox1
 *
 * @param  object   Sender
 *
 * @return void
 */
function CheckBox1OnClick(Sender) {
	if (CheckBox1.Checked) {
		CheckBox2.Checked = true;
	}
}

/**
 * Callback handler for clicks on Checkbox2
 *
 * @param  object   Sender
 *
 * @return void
 */
function CheckBox2OnClick(Sender) {
	if (CheckBox1.Checked) {
		CheckBox2.Checked = true;
	}
	if (CheckBox2.Checked) {
		CheckBox1.Enabled = true;
	}
	else {
		CheckBox1.Enabled = false;
	}
}

/**
 * Callback handler for selecting item in ListBox1
 *
 * @param  object   Sender
 *
 * @return void
 */
function OnSelectItem(Sender) {
	fakeMarker = "";
    for (var i=0;i < Sender.Items.Count;i++) {
        if (Sender.Selected[i] == true) {
        	fakeMarker = KeysListBox1[i];
            break;
        }
    }

	PSForm.ModalResult = mrCancel;	// Close modal window

	if (fakeMarker != "") {

		if (CheckBox2.Checked == true) {
			// Short format
			fakeMarker = RegexReplace(fakeMarker, "\\(.*", "", true);
		}

        fakeMarker = "{{" + fakeMarker + "}}";

		if (CheckBox1.Checked == true) {
			// Expanded mode
			FakerDirect(Sender);
		}
		else {
			Editor.SelText = fakeMarker;
		}
	}
}

/**
 * Callback for sending marker to Webkit for return of translated faker Mustache marker
 *
 * @return void
 */
function RunFakerDirect() {
	var data = fakeMarker + "|" + Script.ReadSetting("Language", "");
	web.Send("com2", data);
}

/**
 * Initialize Webkit and run callback function to return translated faker Mustache marker
 *
 * @param  object   Sender
 *
 * @return void
 */
function FakerDirect(Sender) {
	if (!CreateWebkit("RunFakerDirect")) {
    	RunFakerDirect();
    }
}

/**
 * Callback handler for keypress on ListBox1
 *
 * @param  object   Sender
 * @param  string   key
 *
 * @return string
 */
function OnKeyPress(Sender, key) {
	if (key == chr(13)) {
		// Enter key pressed on GUI item
		OnSelectItem(Sender);
	}
	OnKeyPress2(Sender, key);
}

/**
 * Callback handler for keypress on ListBox1, CheckBox1 and CheckBox2
 *
 * @param  object   Sender
 * @param  string   key
 *
 * @return string
 */
function OnKeyPress2(Sender, key) {
	if (key == chr(27)) {
		// Escape key pressed on GUI item
		PSForm.ModalResult = mrCancel;	// Close modal window
	}
	return key;
}

/**
 * Callback handler for when Webkit window have been created
 *
 * @param  object   Sender
 * @param  string   Browser
 *
 * @return void
 */
function OnWebkitCreated(Sender, Browser) {
  Script.TimeOut(1, "HideWebkitWindow");
}

/**
 * Hide Webkit window
 * Timeout callback handler for when Webkit window have been created
 *
 * @param  object   Sender
 *
 * @return void
 */
function HideWebkitWindow(Sender) {
  win.Caption = "";
  win.Hide();
}

/**
 * Callback handler for when file in Webkit window have been loaded
 *
 * @param  object   Sender 
 * @param  string   Browser
 * @param  object   Frame
 * @param  string   Status
 * @param  string   Res
 *
 * @return void
 */
function OnWebkitLoadEnd(Sender, Browser, Frame, Status, Res) {
  Script.TimeOut(1, CallbackAfterCreate);
}

/**
 * Create modal window for selcting Faker markers
 *
 * @param  function   Callback function
 *
 * @return bool  true if Webkit window has been create false if window already exists
 */
function CreateWebkit(Callback) {
  var Created = false;

  if (win == null) {
     win = new TForm(WeBuilder);
     win.width = 60;
     win.Height = 60;
     win.Caption = "Loading...";
     web = Script.CreateScriptableWebKit(win, Script.Path + "index.html", "OnWebkitCreated");
     web.Subscribe("error", "OnWebkitData"); // Error channel
     web.Subscribe("com1", "OnWebkitData");  // Insert Mustache marker
     web.Subscribe("com2", "OnWebkitData");  // Direct Mustache marker translation
     Web.OnLoadEnd = "OnWebkitLoadEnd";
     web.OnConsoleMessage = "OnWebkitConsoleMessage";
	 web.Webkit.SetBounds(0, 0, win.ClientWidth, win.ClientHeight);
     web.Webkit.Anchors = akLeft + akRight + akTop + akBottom;
     Created = true;
     CallbackAfterCreate = Callback;
     win.Show;
     Document.Activate;
  }
  return Created;
}

/**
 * Callback handler for recieving data from Webkit frame
 *
 * @param  string   channel 
 * @param  string   data 
 *
 * @return void
 */
function OnWebkitData(channel, data) {
	if (channel == "error") {
	    // Error
	    Alert(data);
    }
	else if (channel == "com1") {
		if (Editor.SelText != "") {
			Editor.SelText = data;
		}
        else {
			Editor.Text = data;
        }
	}
    else {
        // Expanded mode
		Editor.SelText = data;
    }
}

/**
 * Passes Console.log() from Webkit frame to WeBuilder
 *
 * @param  object   Sender 
 * @param  string   browser
 * @param  string   message
 * @param  string   source
 * @param  integer  line
 * @param  string   Res
 *
 * @return void
 */
function OnWebkitConsoleMessage(Sender, Browser, message, source, line, Res) {
	Script.Message(_t(line) + ":" + source + ":" + message);
}

/**
 * Callback for sending editor content to Webkit for translateing of faker Mustache markers
 *
 * @return void
 */
function RunFaker() {
	var data = Editor.SelText;
	if (data == "") {
    	data = Editor.Text;
	}
	data += "|" + Script.ReadSetting("Language", "");
	web.Send("com1", data);
}

/**
 * Initialize Webkit and run callback function to translate faker Mustache markers
 *
 * @param  object   Sender
 *
 * @return void
 */
function Faker(Sender) {
	if (!CreateWebkit("RunFaker")) {
    	RunFaker();
    }
}

/**
 * Fired when plugin is ending due to IDE closing or plugin being disabled or uninstalled
 *
 * @return void
 */
function OnExit() {
	if (web != null) {
		delete web;
	}
	if (win != null) {
		delete win;
	}
}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
  alert("FakerWB 1.0 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
Script.ConnectSignal("exit", "OnExit");

var bmp = new TBitmap, act;

LoadFileToBitmap(Script.Path + "faker1-icon.png", bmp);
act = Script.RegisterAction("FakerWB", "Insert faker marker", "", "InsertMaker");
Actions.SetIcon(act, bmp);

LoadFileToBitmap(Script.Path + "faker2-icon.png", bmp);
act = Script.RegisterAction("FakerWB", "Translate faker markers", "", "Faker");
Actions.SetIcon(act, bmp);

LoadFileToBitmap(Script.Path + "help-icon.png", bmp);
act = Script.RegisterAction("FakerWB", "Show Help", "", "ShowHelp");
Actions.SetIcon(act, bmp);

delete bmp;

